<?php
function update_to_log($text) {
    misc::log($text, 'system.update');
}

function dcms_update_backup_create($files) {
    // создание бэкапа обновляемых и удаляемых файлов
    global $dcms;

    $to_backup = array();
    foreach ($files as $key => $value) {
        if (!file_exists(H . '/' . $value)) {
            continue;
        }
        $to_backup[] = H . '/' . $value;
    }


    $zip_file = TMP . '/backup.' . $dcms->version . '.' . TIME . '.zip';

    $zip = new PclZip($zip_file);
    if (!$zip->create($to_backup, PCLZIP_OPT_REMOVE_PATH, H . '/')) {
        update_to_log('Не удалось создать архив с резервной копией. Ошибка: ' . $zip->errorInfo(true));
        @unlink($zip_file);
        return false;
    }


    return $zip_file;
}

function dcms_update_recovery($backup_path) {
    // восстановление бэкапа при неудаче
    $zip = new PclZip($backup_path);
    $zip->extract(PCLZIP_OPT_PATH, H . '/');
}

function dcms_update_extract($zip_path) {
    // распаковка архива файлов обновления
    $tmp_dir = TMP . '/' . passgen();
    $zip = new PclZip($zip_path);
    if ($zip->extract(PCLZIP_OPT_PATH, $tmp_dir . '/')) {
        return $tmp_dir;
    } else {
        //filesystem::rmdir($tmp_dir);
        return false;
    }
}

function dcms_update_delete($to_delete) {
    // удаление файлов, заданных обновлением
    foreach ($to_delete as $path => $hash) {
        $file = H . '/' . $path;
        if (!file_exists($file)) {
            continue;
        }

        if (!@unlink($file)) {
            return false;
        }
    }
    return true;
}

function dcms_update_update($updir, $to_update) {
    // замена файлов
    foreach ($to_update as $file) {
        if (!@copy($updir . '/' . $file, H . '/' . $file)) {

            update_to_log('Не удалось обновить (сохранить) файл ' . $file);
            return false;
        }
        @chmod(H . '/' . $file, filesystem::getChmodToWrite());
    }
    return true;
}

function dcms_update_check($updir) {
    // проверка содержимого пакета обновления
    if (!file_exists($updir . '/to_delete.ini')) {
        return false;
    }
    if (!file_exists($updir . '/to_update.ini')) {
        return false;
    }
    if (!file_exists($updir . '/version.ini')) {
        return false;
    }


    $to_update = keyvalue::read($updir . '/to_update.ini');

    foreach ($to_update as $fname) {
        if (!file_exists($updir . '/' . $fname)) {
            // отсутствует заявленный в обновлении файл
            return false;
        }
    }


    return true;
}

function dcms_update_check_version($updir) {
    global $dcms;
    // проверка соответствия обновления текущей версии движка
    $version = keyvalue::read($updir . '/version.ini');

    if (empty($version['from']) || $dcms->version !== $version['from']) {
        // пакет обновления не предназначен для этой версии
        return false;
    }
    return true;
}

function dcms_update_sql() {
    // обновление структуры таблиц в базе данных


    $tables_exists = new tables();
    $table_files = (array) glob(H . '/sys/preinstall/base.create.*.ini');
    $tables = array();
    foreach ($table_files as $table_file) {
        preg_match('#base.create\.(.+)\.ini#ui', $table_file, $m);
        $tables[] = $m[1];
    }


    foreach ($tables as $table) {

        $tab = new table_structure(H . '/sys/preinstall/base.create.' . $table . '.ini');
        // если такая таблица уже существует, то получаем запрос на ее изменение
        if (in_array($table, $tables_exists->tables)) {
            $tab_old = new table_structure();
            $tab_old->loadFromBase($table);
            $sql = $tab_old->getSQLQueryChange($tab);
        } else {
            $sql = $tab->getSQLQueryCreate();
        }

        mysql_query($sql);
    }
}

function dcms_update_start($zip_path) {
    
    set_time_limit(600); // время на обновление движка ставим 10 минут. Этого более чем достаточно для выполнения всех действий.
    ignore_user_abort(); // нельзя прерывать процесс обновления даже если пользователем он отменен.

    update_to_log('Начинаем процесс обновления');
    update_to_log('Файл обновления: ' . $zip_path);

    if (!$update_dir_path = dcms_update_extract($zip_path)) {
        // если не можем распаковать обновление, то нафиг оно нам...
        update_to_log('Не удалось распаковать архив');
        return false;
    }
    update_to_log('Архив успешно распакован');

    if (!dcms_update_check($update_dir_path)) {
        // пакет обновления не полон или содержит ошибки
        update_to_log('Архив не является пакетом обновления');
        filesystem::rmdir($update_dir_path);
        return false;
    }
    update_to_log('Содержимое архива проверено');


    if (!dcms_update_check_version($update_dir_path)) {
        // пакет обновления не подходит к данной версии
        update_to_log('Пакет обновления не предназначен для данной версии');
        filesystem::rmdir($update_dir_path);
        return false;
    }

    update_to_log('Версия пакета обновления соответствует версии системы');


    $to_delete = (array) keyvalue::read($update_dir_path . '/to_delete.ini');
    $files_to_backup = $to_update = (array) keyvalue::read($update_dir_path . '/to_update.ini');

    foreach ($to_delete as $file => $hash) {
        $files_to_backup[] = $file;
    }



    update_to_log('Создаем резервную копию обновляемых файлов');
    if (!$backup_path = dcms_update_backup_create($files_to_backup)) {
        // если бэкап не создали, то обновлять опасно.
        update_to_log('При создании резервной копии возникли ошибки. Обновление отменено.');
        filesystem::rmdir($update_dir_path);
        return false;
    }
    update_to_log('Резервная копия успешно создана');



    update_to_log('Удаляем старые файлы системы');
    if (!dcms_update_delete($to_delete)) {
        // если во время удаления файлов возникла ошибка, то продолжать не будем, а лучше восстановим как было
        update_to_log('При удалении файлов возникли ошибки. Восстанавливаем данные из резервной копии');
        dcms_update_recovery($backup_path);
        filesystem::rmdir($update_dir_path);
        update_to_log('Данные из резервной копии восстановлены. Обновление отменено.');
        return false;
    }
    update_to_log('Файлы успешно удалены');


    update_to_log('Начинаем процесс обновления');
    if (!dcms_update_update($update_dir_path, $to_update)) {
        // если какой-то файл обновить не удалось, то восстанавливаемся из бэкапа
        update_to_log('Не удалось обновить некоторые файлы. Восстанавливаем данные из резервной копии');
        dcms_update_recovery($backup_path);
        filesystem::rmdir($update_dir_path);
        update_to_log('Данные из резервной копии восстановлены. Обновление отменено.');
        return false;
    }
    update_to_log('Обновление файлов движка произведено успешно');

    // обновляем Базу данных
    update_to_log('Обновляем структуру базы данных');
    dcms_update_sql();
    update_to_log('Структура базы данных успешно обновлена');

    // зачищаем временные файлы обновлений
    update_to_log('Удаляем временные файлы пакета обновлений');
    filesystem::rmdir($update_dir_path);

    update_to_log('Обновление движка успешно завершено');
    return true;
}

function dcms_update($zip_path) {
    cache::set('system.update.work', true, 600);

    $return = dcms_update_start($zip_path);

    cache::set('system.update.work', false);
    return $return;
}


function dcms_update_check_new_version() {
    $new_version = false;
    $curl = new http_client('http://dcms.su/build/config.ini');
    $config_content = $curl->getContent();

    if (!$config_content) {
        return false;
    }

    $tmp_file = TMP . '/' . passgen() . '.ini';
    if (!@file_put_contents($tmp_file, $config_content)) {
        return false;
    }

    if (!$conf = ini::read($tmp_file)) {
        @unlink($tmp_file);
        return false;
    }

    if (empty($conf['version_last']) || empty($conf['build_num'])) {
        return false;
    }

    @unlink($tmp_file);

    return $conf['version_last'] . '.' . $conf['build_num'];
}

function dcms_update_check_new_update($no_cache = false) {
    if (!$no_cache && $new_ver = cache::get('system.update.new_version')) {
        return $new_ver;
    }
    $new_ver = dcms_update_check_new_version();

    cache::set('system.update.new_version', $new_ver, 10);


    return $new_ver;
}

function dcms_update_auto() {
    global $dcms;
    $newversion = dcms_update_check_new_update();
    $curl = new http_client('http://dcms.su/build/updates.php?from=' . $dcms->version . '&to=' . $newversion);
    $tmp_file = TMP . '/' . passgen() . '.zip';
    if ($curl->save_content($tmp_file)) {
        $return = dcms_update($tmp_file);
       // @unlink($tmp_file);
        return $return;
    } else {
        return false;
    }
}

?>
