
<?php

class http_client {

    var $ua = 'DCMSSeven/httpClient (dcms.su)'; // USER AGENT
    var $timeout = 20; // таймаут запроса (сек)
    var $referer = '';
    protected $_url;
    protected $_post = array(); // массив с POST
    protected $_cookie = array(); // массив с COOKIE
    protected $_sock = false;
    var $errn;
    var $errs;

    /**
     * curl::__construct()
     * Инициализация
     * @param string $url Запрашиваемый адрес
     */
    function __construct($url) {
        $this->_url = $url;
        $this->dataReset();
    }

    public function dataReset() {
        $this->_post = array();
        $this->_cookie = array();
        $this->_sock = false;
        $this->errn = '';
        $this->errs = '';
    }

    protected function _connect() {
        $purl = @parse_url($this->_url);
        if (!empty($purl['host'])) {
            $port = empty($purl['port']) ? 80 : $purl['port'];

            if (empty($purl['scheme'])) {
                $scheme = '';
            } elseif ($purl['scheme'] == 'https') {
                $scheme = 'ssl://';
            }else{
                $scheme = '';
            }
            

            if (!$this->_sock = @fsockopen($scheme . $purl['host'], $port, $this->errn, $this->errs, $this->timeout)) {
                misc::log($scheme . $purl['host'] . ' - Не удалось подключиться', 'http_client');
            } else {
                stream_set_timeout($this->_sock, $this->timeout);
                misc::log($scheme . $purl['host'] . ' - Успешно подлючились', 'http_client');

                return true;
            }
        }
    }

    protected function _disconnect() {
        fclose($this->_sock);
        misc::log('Сокет закрыт', 'http_client');
        //$this->dataReset();
    }

    /**
     * curl::set_post()
     * Установка переменной POST
     * @param string $name Название
     * @param string $value Значение
     * @return bool
     */
    public function set_post($name, $value = null) {
        $this->_post[] = urlencode($name) . '=' . urlencode($value);
    }

    /**
     * curl::set_cookie()
     * Установка переменной COOKIE
     * @param string $name Название
     * @param string $value Значение
     * @return bool
     */
    public function set_cookie($name, $value = null) {
        //$this->_cookie[] = $name . '=' . $value;
        $this->_cookie[] = urlencode($name) . '=' . urlencode($value);
    }

    /**
     * curl::save_content()
     * Сохраняет полученную информацию в файл
     * @param string $file_path Путь к файлу
     * @return bool
     */
    public function save_content($file_path) {
        misc::log('Запрос на сохранение файла', 'http_client');

        if (!$fo = @fopen($file_path, 'wb')) {
            misc::log('Не удалось открыть указатель ресурса', 'http_client');
            return false;
        }
        if (!$this->_connect()) {
            misc::log('Соединение разорвано', 'http_client');
            return false;
        }



        fputs($this->_sock, $this->getOutputHeaders());
        // echo nl2br($this -> _getOutputHeaders());
        // заголовки

        $headers = '';
        while (!feof($this->_sock)) {
            $data = fgets($this->_sock, 2048);
            if ($data == "\r\n") {
                break;
            }
            $headers .= $data;
        }

        // misc::log($headers, 'http_client');
        // сохраняем контент
        while (!feof($this->_sock)) {
            if ($data = fgets($this->_sock, 4096)) {
                if (!@fwrite($fo, $data)) {
                    return false;
                }
            }
        }
        misc::log('Файл успешно получен и сохранен', 'http_client');
        fclose($fo);
        $this->_disconnect();
        return true;
    }

    /**
     * curl::getHeaders()
     * Получение заголовков
     * @return string
     */
    public function getHeaders() {
        return $this->get_headers();
    }

    /**
     * curl::getContent()
     * Получение содержимого
     * @return string
     */
    public function getContent() {
        return $this->get_content();
    }

    /**
     * curl::get_content()
     * Получение содержимого
     * @return string
     */
    public function get_content() {
        misc::log('Запрос на получение контента', 'http_client');
        if (!$this->_connect()) {
            misc::log('Соединение разорвано', 'http_client');
            return false;
        }

        fputs($this->_sock, $this->getOutputHeaders());
        // echo nl2br($this -> _getOutputHeaders());
        // заголовки

        $headers = '';
        while (!feof($this->_sock)) {
            $data = fgets($this->_sock, 2048);
            if ($data == "\r\n") {
                break;
            }
            $headers .= $data;
        }

        // misc::log($headers, 'http_client');
        // сохраняем контент
        $content = '';
        while (!feof($this->_sock)) {
            if ($data = fgets($this->_sock, 4096)) {
                $content .= $data;
            }
        }
        
        
        misc::log('Контент успешно получен ('.misc::size_data(strlen($content)).')', 'http_client');
        // echo $content;
        $this->_disconnect();
        return $content;
    }

    /**
     * curl::get_headers()
     * Получение заголовков
     * @return string
     */
    public function get_headers() {
        misc::log('Запрос на получение заголовков', 'http_client');
        if (!$this->_connect()) {
            misc::log('Соединение разорвано', 'http_client');
            return false;
        }



        fputs($this->_sock, $this->getOutputHeaders());
        $headers = '';
        while (!feof($this->_sock)) {
            $data = fgets($this->_sock, 2048);
            if ($data == "\r\n") {
                break;
            }
            $headers .= $data;
        }
        misc::log('Заголовки успешно получены', 'http_client');
        $this->_disconnect();
        //misc::log($headers, 'http_client');
        return $headers;
    }

    /**
     * curl::_getOutputHeaders()
     * Формирование отправляемого заголовка
     * @return string
     */
    function getOutputHeaders() {
        $headers = array();
        $purl = @parse_url($this->_url);
        $path = empty($purl['path']) ? '/' : $purl['path'];
        $query = empty($purl['query']) ? '' : '?' . $purl['query'];
        $headers[] = ($this->_post ? 'POST' : 'GET') . ' ' . $path . $query . ' HTTP/1.0';
        // Хост
        $host = empty($purl['host']) ? '' : $purl['host'];
        $headers[] = 'Host: ' . $host;


        // Реферер
        if ($this->referer) {
            $headers[] = 'Referer: ' . $this->referer;
        }
        // Печеньки
        if ($this->_cookie) {
            $headers[] = 'Cookie: ' . implode(';', $this->_cookie);
        }
        // Браузер
        if ($this->ua) {
            $headers[] = 'User-Agent: ' . $this->ua;
        }


        // формируем POST данные

        if ($this->_post) {
            $post_data = implode('&', $this->_post);
            $headers[] = 'Content-Type: application/x-www-form-urlencoded';
            $headers[] = 'Content-Length: ' . strlen($post_data);
        }
        // HTTP авторизация
        if (isset($purl['user']) && isset($purl['pass'])) {
            $headers[] = 'Authorization: Basic ' . base64_encode($purl['user'] . ':' . $purl['pass']);
        }
        // поддержание подключения (позволяет отправлять несколько запросов за одно подключение)
        $headers[] = 'Connection: Close';
        $header = implode("\r\n", $headers) . "\r\n\r\n";
        // данные, переданные с помощью POST
        if (!empty($post_data)) {
            $header .= $post_data;
        }

        //misc::log($header, 'http_client');
        return $header;
    }

}

?>