<?php

class language_pack {

    protected $_isset = false;
    protected $_loaded = false;
    protected $_data = array();
    protected $_default = array();
    protected $_save_default = false;
    protected $_script = 'system';
    var $code;
    var $name;
    var $enname;

    function __construct($code) {
        $this->code = $code;

        if (languages::exists($this->code)) {
            $this->_isset = true;
        }

        $config = languages::getConfig($this->code);
        $this->enname = $config['enname']; // название языка на английском
        $this->name = $config['name']; // локальное название языка

        $this->_script = str_replace(array('/', '\\'), '_', $_SERVER['SCRIPT_NAME']);
        $this->_default = keyvalue::read(H . '/sys/languages/for_translate.lng');
    }

    protected function _load() {
        // загрузка всех данных языкового пакета

        if ($data = cache::get('language.data.' . $this->code)) {
            return $data;
        }

        $data = array();
        $lngs = (array) glob(H . '/sys/languages/' . $this->code . '/*.lng');
        foreach ($lngs as $file) {

            $data[basename($file, '.lng')] = keyvalue::read($file);
        }
        cache::set('language.data.' . $this->code, $data, 60);
        return $data;
    }

    protected function _addString($str) {
        if ($this->code == 'default') {
            return false;
        }

        $this->_data[$this->_script][$str] = $str;
        keyvalue::save(H . '/sys/languages/' . $this->code . '/' . $this->_script . '.lng', $this->_data[$this->_script]);
        $this->clearCache();
    }

   
    
    
    public function getString($str) {
        if (empty($this->_default[$str])) {
            // пополнение общего списка переводимых фраз
            $this->_default[$str] = $str;
            $this->_save_default = true;
        }

        if (!$this->_isset) {
            // если языковой пакет не загружен, то возвращаем строку в исходном виде
            return $str;
        }


        if ($this->code == 'default') {
            // языковой пакет по-умолчанию
            return $str;
        }







        if (!$this->_loaded) {
            // если языковой пакет еще не загружен, то загружаем
            $this->_loaded = true;
            $this->_data = $this->_load();
        }

        if (isset($this->_data[$this->_script][$str])) {
            // перевод найден в спец. словаре
            return $this->_data[$this->_script][$str];
        }

        if (isset($this->_data['system'][$str])) {
            // перевод найден в системном словаре
            return $this->_data['system'][$str];
        }



        $this->_addString($str);
        return $str;
    }

    function clearCache() {
        cache::set('language.data.' . $this->code, false);
    }

    function __destruct() {
        if ($this->_save_default) {
            keyvalue::save(H . '/sys/languages/for_translate.lng', $this->_default);
        }
    }

}

?>
